<?php

namespace QUI\NodeDev;

use DirectoryIterator;
use QUI\System\Log;

use function array_merge;
use function chdir;
use function copy;
use function dirname;
use function file_exists;
use function file_get_contents;
use function is_string;
use function json_decode;
use function shell_exec;
use function unlink;

/**
 * Generates an import map for the given node_modules folder
 *
 * tried to implement: https://nodejs.org/docs/latest-v16.x/api/esm.html#esm_resolution_algorithm
 */
class ImportMapGenerator
{
    private string $baseDir = '';

    public function __construct(string $baseDir)
    {
        $realBaseDir = realpath($baseDir);

        if ($realBaseDir !== false) {
            $this->baseDir = $realBaseDir;
        }
    }

    /**
     * Generates an import map for the given node_modules folder
     *
     * Tries to implement the resolution algorithm as described here:
     * https://nodejs.org/docs/latest-v16.x/api/esm.html#esm_resolution_algorithm
     *
     * Also adds a special case for Vue.js, to ensure that the correct file is loaded.
     *
     * @return array The generated import map
     */
    public function generateImportMap(): array
    {
        $importMapFile = $this->baseDir . '/importmap.json';

        if (file_exists($importMapFile)) {
            return json_decode(file_get_contents($importMapFile), true);
        }

        // generate map
        $generateJsFile = $this->baseDir . '/generateImportMap.js';

        if (file_exists($generateJsFile)) {
            unlink($generateJsFile);
        }

        try {
            $Console = new Console();
            $nodeCommand = $Console->getNodeJsCommand();
            $currentDir = getcwd();
        } catch (\Exception $exception) {
            Log::addError($exception->getMessage());
            return [];
        }

        copy(dirname(__FILE__, 4) . '/scripts/generateImportMap.js', $generateJsFile);

        chdir($this->baseDir);
        $result = shell_exec($nodeCommand . ' generateImportMap.js');
        chdir($currentDir);

        if (file_exists($importMapFile)) {
            return json_decode(file_get_contents($importMapFile), true);
        }

        Log::addError('Could not create import map', [
            'command' => $result
        ]);

        return [];
    }
}
